@@ -2,7 +2,7 @@ package WebService::ILS;
 
 use Modern::Perl;
 
-our $VERSION = "0.01";
+our $VERSION = "0.02";
 
 =encoding utf-8
 
@@ -168,6 +168,10 @@ sub params_check {
 =item C<user_agent_params> => LWP::UserAgent constructor params
                               so you don't need to create user agent yourself
 
+=item C<access_token>      => as returned from the provider authentication system
+
+=item C<access_token_type> => as returned from the provider authentication system
+
 =back
 
 These are also read-only attributes
@@ -179,13 +183,15 @@ Not all of client/library params are required for all service providers.
 use Class::Tiny qw(
     user_agent
     client_id client_secret library_id
-    _access_token _access_token_type
+    access_token access_token_type
 );
 
 __PACKAGE__->_set_param_spec({
     client_id         => { required => 1, defined => 1 },
     client_secret     => { required => 1, defined => 1 },
     library_id        => { required => 0, defined => 1 },
+    access_token      => { required => 0 },
+    access_token_type => { required => 0 },
     user_agent        => { required => 0 },
     user_agent_params => { required => 0 },
 });
@@ -330,6 +336,27 @@ Provider authentication API is used to get an authorized session.
 
 =head3 auth_by_user_id($user_id, $password)
 
+An example:
+
+    my $ils = WebService::ILS::Provider({
+        client_id => $client_id,
+        client_secret => $client_secret,
+    });
+    eval { $ils->auth_by_user_id( $user_id, $password ) };
+    if ($@) { some_error_handling(); return; }
+    $session{ils_access_token} = $ils->access_token;
+    $session{ils_access_token_type} = $ils->access_token;
+    ...
+    Somewhere else in your app:
+    my $ils = WebService::ILS::Provider({
+        client_id => $client_id,
+        client_secret => $client_secret,
+        access_token => $session{ils_access_token},
+        access_token_type => $session{ils_access_token_type},
+    });
+ 
+    my $checkouts = $ils->checkouts;
+
 =head2 Authentication at the provider
 
 User is redirected to the provider authentication url, and after
@@ -368,15 +395,11 @@ An example:
     local $@;
     eval { $ils->auth_by_token( $auth_token ) };
     if ($@) { some_error_handling(); return; }
-    $session{ils_token} = $auth_token
+    $session{ils_access_token} = $ils->access_token;
+    $session{ils_access_token_type} = $ils->access_token;
     ...
     Somewhere else in your app:
-    my $auth_token = $session{ils_token}
-    local $@;
-    eval { $ils->auth_by_token( $auth_token ) };
-    if ($@) { some_error_handling(); return; }
-
-    my $checkouts = $ils->checkouts;
+    passing access token to the constructor as above
 
 =cut
 
@@ -563,7 +586,7 @@ sub _request_with_auth {
     my $self = shift;
     my $request = shift or croak "No request";
 
-    my $has_token = $self->_access_token;
+    my $has_token = $self->access_token;
     my $response = $self->_request_with_token($request);
     # token expired?
     $response = $self->_request_with_token($request, "FRESH TOKEN")
@@ -591,8 +614,8 @@ sub _request_access_token {
             my ($data) = @_;
 
             my $token = $data->{access_token} or die "No access token\n";
-            $self->_access_token($token);
-            $self->_access_token_type($data->{token_type} || 'Bearer');
+            $self->access_token($token);
+            $self->access_token_type($data->{token_type} || 'Bearer');
             return $data;
         },
         sub {
@@ -613,14 +636,14 @@ sub _request_with_token {
     my $request = shift or croak "No request";
     my $force_fresh = shift;
 
-    my $token = $force_fresh ? $self->_access_token : undef;
+    my $token = $force_fresh ? undef : $self->access_token;
     unless ($token) {
         my $request = $self->make_access_token_request;
         $self->_request_access_token($request);
-        $token = $self->_access_token;
+        $token = $self->access_token;
     }
     die "No access token" unless $token;
-    my $token_type = $self->_access_token_type;
+    my $token_type = $self->access_token_type;
 
     $request->header( Authorization => "$token_type $token" );
     return $self->user_agent->request( $request );
