@@ -46,7 +46,7 @@ use constant AUTH_URL => OAUTH_BASE_URL . 'auth';
 
 =over 12
 
-=item C<authorization_name> => auth domain as provided by OverDrive
+=item C<auth_token> => auth token as previously obtained
 
 =back
 
@@ -59,12 +59,18 @@ use Class::Tiny qw(
     _circulation_api_url => sub { $_[0]->test ? TEST_CIRCULATION_API_URL : CIRCULATION_API_URL },
 };
 
+__PACKAGE__->_set_param_spec({
+    auth_token      => { required => 0 },
+});
+
 =head1 INDIVIDUAL USER AUTHENTICATION METHODS
 
 =head2 auth_by_user_id ($user_id, $password, $website_id, $authorization_name)
 
 C<website_id> and C<authorization_name> (domain) are provided by OverDrive
 
+=head3 Returns (access_token, access_token_type) or access_token
+
 =cut
 
 sub auth_by_user_id {
@@ -81,7 +87,7 @@ sub auth_by_user_id {
     $self->password($password);
     $self->website_id($website_id);
     $self->authorization_name($authorization_name);
-    return 1;
+    return wantarray ? ($self->access_token, $self->access_token_type) : $self->access_token;
 }
 
 sub _make_access_token_by_user_id_request {
@@ -126,16 +132,20 @@ An example:
         or some_error_handling(), return;
     # my $state = $req->param( $overdrive->state_token_param_name )...
     local $@;
-    my $auth_token = eval { $overdrive->auth_by_code( $auth_code ) };
+    eval { $overdrive->auth_by_code( $auth_code ) };
     if ($@) { some_error_handling(); return; }
-    $session{overdrive_token} = $auth_token
+    $session{overdrive_access_token} = $access_token;
+    $session{overdrive_access_token_type} = $access_token_type;
+    $session{overdrive_auth_token} = $auth_token;
     ...
     Somewhere else in your app:
-    my $auth_token = $session{overdrive_token}
-    local $@;
-    eval { $overdrive->auth_by_token( $auth_token ) };
-    if ($@) { some_error_handling(); return; }
-
+    my $ils = WebService::ILS::Provider({
+        client_id => $client_id,
+        client_secret => $client_secret,
+        access_token => $session{overdrive_access_token},
+        access_token_type => $session{overdrive_access_token_type},
+        auth_token = $session{overdrive_auth_token}
+    });
     my $checkouts = $overdrive->checkouts;
 
 =head2 auth_url ($redirect_uri, $state_token)
@@ -184,6 +194,8 @@ use constant state_token_param_name => "code";
 
 =head2 auth_by_code ($provider_code, $redirect_uri)
 
+=head3 Returns (access_token, access_token_type, auth_token) or access_token
+
 =cut
 
 sub auth_by_code {
@@ -198,24 +210,25 @@ sub auth_by_code {
         code => $code,
         redirect_uri => $redirect_uri,
     } );
-    my $data = $self->_request_access_token($request);
-    my $auth_token = $self->auth_token or die "No auth token\n"; # XXX Paranoid
-    return $auth_token;
+    $self->_request_access_token($request);
+    return wantarray ? ($self->access_token, $self->access_token_type, $self->auth_token) : $self->access_token;
 }
 
 =head2 auth_by_token ($provider_token)
 
+=head3 Returns (access_token, access_token_type, auth_token) or access_token
+
 =cut
 
 sub auth_by_token {
     my $self = shift;
     my $auth_token = shift or croak "No authorization token";
 
+    $self->auth_token($auth_token);
     my $request = $self->_make_access_token_by_auth_token_request($auth_token);
     $self->_request_access_token($request);
 
-    $self->auth_token($auth_token);
-    return 1;
+    return wantarray ? ($self->access_token, $self->access_token_type, $self->auth_token) : $self->access_token;
 }
 
 sub _make_access_token_by_auth_token_request {
